import 'package:cl_appbase/api.dart';
import 'package:cl_appbase/appbase.dart';

import '../api.dart';
import '../api_resource.dart';

class ListHubResource<TData extends TransferObject, TId>
    extends ListApiResource<TData, TId> implements Collection {
  const ListHubResource(ApiBase api, String endpointSuffix, factory)
      : super(api, endpointSuffix, factory);

  Future<TData> getMetaData<TData>(String key,
      {Map<String, dynamic>? params, Map<String, String>? query}) async {
    final response = await request<Map<String, dynamic>, Map<String, dynamic>>(
        api.baseUrl, '$endpointSuffix/\$$key', params, ApiRequestMethod.GET,
        headers: api.getRequestHeaders(), query: query);
    response.throwOnError();
    return response.data[key] as TData;
  }

  /// Implements Collection.getItems
  @override
  Future<List<TData>> getItems(int offset, int limit) async =>
      await getList(offset, limit);

  /// Implements Collection.getSize
  @override
  Future<int> getSize() async => await getMetaData('count');

  /// Creates a wrapper object that contains default route params and
  /// query params
  ListHubResource<TData, TId> withParams(
          {Map<String, dynamic>? params, Map<String, String>? query}) =>
      _ListHubDelegate(api, endpointSuffix, factory,
          defaultParams: params, defaultQuery: query);
}

class SingleObjectHubResource extends SingleObjectApiResource {
  SingleObjectHubResource(ApiBase api, String endpointSuffix, factory)
      : super(api, endpointSuffix, factory);

  Future<TData> getMetaData<TData>(String key) async {
    final response = await request<Map<String, dynamic>, Map<String, dynamic>>(
        api.baseUrl, '$endpointSuffix/\$$key', null, ApiRequestMethod.GET,
        headers: api.getRequestHeaders());
    response.throwOnError();
    return response.data[key] as TData;
  }
}

class _ListHubDelegate<TData extends TransferObject, TId>
    extends ListHubResource<TData, TId> {
  final Map<String, dynamic>? defaultParams;
  final Map<String, String>? defaultQuery;

  const _ListHubDelegate(ApiBase api, String endpointSuffix, factory,
      {this.defaultParams, this.defaultQuery})
      : super(api, endpointSuffix, factory);

  @override
  Future<List<TData>> getList(int offset, int limit,
          {Map<String, dynamic>? params, Map<String, String>? query}) async =>
      super.getList(offset, limit,
          params: _params(params), query: _query(query));

  @override
  Future<TData> getMetaData<TData>(String key,
          {Map<String, dynamic>? params, Map<String, String>? query}) async =>
      await super
          .getMetaData(key, params: _params(params), query: _query(query));

  @override
  Future<TData> getElement(TId id,
          {Map<String, dynamic>? params, Map<String, String>? query}) async =>
      await super.getElement(id, params: _params(params), query: _query(query));

  Future<TData> postElement(TData element,
          {Map<String, dynamic>? params, Map<String, String>? query}) async =>
      await super
          .postElement(element, params: _params(params), query: _query(query));

  Future<TData> putElement(TData element,
          {Map<String, dynamic>? params, Map<String, String>? query}) async =>
      await super
          .putElement(element, params: _params(params), query: _query(query));

  Future<TData> patchElement(TData element,
          {Map<String, dynamic>? params, Map<String, String>? query}) async =>
      await super
          .patchElement(element, params: _params(params), query: _query(query));

  Future<void> deleteElement(TId id,
          {Map<String, dynamic>? params, Map<String, String>? query}) async =>
      await super
          .deleteElement(id, params: _params(params), query: _query(query));

  Map<String, dynamic>? _params(Map<String, dynamic>? params) {
    if (params != null && defaultParams != null) {
      return Map.of(defaultParams!)..addAll(params);
    } else {
      return defaultParams ?? params;
    }
  }

  Map<String, String>? _query(Map<String, String>? query) {
    if (query != null && defaultQuery != null) {
      return Map.of(defaultQuery!)..addAll(query);
    } else {
      return defaultQuery ?? query;
    }
  }
}
